<?php

   /**
    *
    * class for Twitter API
    *
    */

    class ueldo_twitter_api {

       /**
        *
        * consumer oauth key
        *
        */

        private $consumer_key = '';

       /**
        *
        * consumer oauth secret key
        *
        */

        private $consumer_secret_key = '';

       /**
        *
        * current bearer token
        *
        */

        private $bearer_token = false;

       /**
        *
        * transient expiration time
        *
        */

        private $transient_expiration = 120;

       /**
        *
        * update consumer keys
        *
        */

        public function set_key( $type, $value ) {

            # action depends on key type
            switch( $type ) {

                # set oauth key
                case 'oauth_key': $this -> consumer_key = $value; break;

                # set oauth secret key
                case 'oauth_secret_key': $this -> consumer_secret_key = $value; break;
            }
        }

       /**
        *
        * get the bearer token from Twitter API
        *
        */

        private function get_new_bearer_token() {

            # get response from Twitter API
            $response = wp_remote_post( 'https://api.twitter.com/oauth2/token', array(
                'method' => 'POST',
                'timeout' => 5,
                'redirection' => 5,
                'httpversion' => '1.0',
                'blocking' => true,
                'headers' => array(
                    'Authorization' => 'Basic '. base64_encode( esc_attr( $this -> consumer_key ) .':'. esc_attr( $this -> consumer_secret_key ) ),
                    'Content-Type' => 'application/x-www-form-urlencoded;charset=UTF-8',
                    'Accept-Encoding' => 'gzip'
                ),
                'body' => array( 'grant_type' => 'client_credentials' ),
                'cookies' => array()
            ));

            # do we have correct response?
            if( is_wp_error( $response ) || $response['response']['code'] != 200 ) return( false );

            # decode json
            $result = json_decode( $response['body'] );

            # update bearer token option
            set_transient( 'ueldo_twitter_api_bearer_token', esc_attr( $result -> access_token ), $this -> transient_expiration );

            # return result
            return( esc_attr( $result -> access_token ) );
        }

       /**
        *
        * send query to Twitter API
        *
        */

        private function query( $query ) {

            # do we have bearer token set?
            if( $this -> bearer_token == false ) {

                # try to get bearer token from transient
                $transient_bearer_token = get_transient( 'ueldo_twitter_api_bearer_token' );

                # is it possible to get bearer token from transient?
                $this -> bearer_token = $transient_bearer_token == false ? $this -> get_new_bearer_token() : $transient_bearer_token;
            }

            # get response from Twitter API
            $response = @wp_remote_get( 'https://api.twitter.com/1.1/statuses/user_timeline.json?'. esc_attr( $query ), array(
                'method' => 'GET',
                'timeout' => 5,
                'redirection' => 5,
                'httpversion' => '1.0',
                'blocking' => true,
                'compress' => false,
                'decompress' => true,
                'headers' => array(
                    'Authorization' => 'Bearer '. esc_attr( $this -> bearer_token ),
                    'Accept-Encoding' => 'gzip'
                ),
                'body' => null,
                'cookies' => array()
            ));

            # do we have correct response?
            if( is_wp_error( $response ) || $response['response']['code'] != 200 ) return( false );

            # return decoded json
            return( json_decode( $response['body'] ) );
        }

       /**
        *
        * translate tweet contents
        *
        */

        private function translate_content( $content, $data ) {

            # replace hashtags
            if( count( $data -> hashtags ) > 0 ) {

                # walk for each found hashtag
                foreach( $data -> hashtags as $hashtag ) {

                    # update content
                    $content = str_replace( '#'. esc_html( $hashtag -> text ), '<a href="'. esc_url( 'https://twitter.com/search?q=%23'. esc_html( $hashtag -> text ) .'&src=hash' ) .'">#'. esc_html( $hashtag -> text ) .'</a>', $content );
                }
            }

            # replace urls
            if( count( $data -> urls ) > 0 ) {

                # walk for each found url
                foreach( $data -> urls as $url ) {

                    # update content
                    $content = str_replace( esc_url( $url -> url ), '<a href="'. esc_url( $url -> expanded_url ) .'">'. esc_html( $url -> display_url ) .'</a>', $content );
                }
            }

            # replace user mentions
            if( count( $data -> user_mentions ) > 0 ) {

                # walk for each found mention
                foreach( $data -> user_mentions as $user ) {

                    # update content
                    $content = str_replace( '@'. esc_html( $user -> screen_name ), '<a href="'. esc_url( 'https://twitter.com/'. esc_html( $user -> screen_name ) ) .'">@'. esc_html( $user -> screen_name ) .'</a>', $content );
                }
            }

            # return content
            return $content;
        }

       /**
        *
        * get single twitter tweet
        *
        */

        public function get_single_tweet( $username, $on_stack = 1, $expiration = 0 ) {

            # try to find a tweet saved in transient
            $transient_tweet = get_transient( 'ueldo_twitter_api_single_tweet_'. esc_attr( $on_stack ) .'_on_stack' );

            # do we have tweet saved in transient?
            if( $transient_tweet != false ) return( $transient_tweet );

            # get last tweets
            $tweets = $this -> query( 'screen_name='. esc_attr( $username ) .'&count='. esc_attr( $on_stack ) );

            # do we have the twetts get?
            if( $tweets == false ) return( false );

            # result array
            $result = array();

            # walk for each tweet
            foreach( $tweets as $tweet ) {

                # update result array
                $result[] = array(
                    'id' => $tweet -> id,
                    'content' => $this -> translate_content( $tweet -> text, $tweet -> entities ),
                    'date' => date_format( date_create( $tweet -> created_at ), 'U' )
                );
            }

            # get searched tweet
            $result = isset( $result[( $on_stack - 1 )] ) ? $result[( $on_stack - 1 )] : false;

            # do we have tweet?
            if( $result != false ) {

                # save tweet in transient
                if( $expiration > 0 ) set_transient( 'ueldo_twitter_api_single_tweet_'. esc_attr( $on_stack ) .'_on_stack', $result, $expiration );
            }

            # retun result
            return( $result );
        }

       /**
        *
        * end of class.
        *
        */
    }

   /**
    *
    * end of twitter api.
    *
    */

?>